@@ -1,26 +1,11 @@
 /*
     This file is part of KDE.
 
-    Copyright (c) 2008 Cornelius Schumacher <schumacher@kde.org>
-    Copyright (c) 2010 Sebastian Kügler <sebas@kde.org>
-    Copyright (c) 2011 Laszlo Papp <djszapi@archlinux.us>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) version 3, or any
-    later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), which shall
-    act as a proxy defined in Section 6 of version 3 of the license.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-FileCopyrightText: 2008 Cornelius Schumacher <schumacher@kde.org>
+    SPDX-FileCopyrightText: 2010 Sebastian Kügler <sebas@kde.org>
+    SPDX-FileCopyrightText: 2011 Laszlo Papp <djszapi@archlinux.us>
 
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #include "provider.h"
@@ -81,6 +66,7 @@
 #include "topicparser.h"
 #include "itemjob.h"
 #include "listjob.h"
+#include "version.h"
 
 #include <QStringList>
 #include <QNetworkAccessManager>
@@ -89,6 +75,7 @@
 #include <QUrlQuery>
 #include <QNetworkReply>
 #include <QFile>
+#include <QCoreApplication>
 
 using namespace Attica;
 
@@ -122,6 +109,7 @@ public:
     QString m_commentVersion;
     QString m_registerUrl;
     PlatformDependent *m_internals;
+    QString m_additionalAgentInformation;
 
     Private()
         : m_internals(nullptr)
@@ -144,13 +132,15 @@ public:
         , m_commentVersion(other.m_commentVersion)
         , m_registerUrl(other.m_registerUrl)
         , m_internals(other.m_internals)
+        , m_additionalAgentInformation(other.m_additionalAgentInformation)
     {
     }
 
     Private(PlatformDependent *internals, const QUrl &baseUrl, const QString &name, const QUrl &icon,
             const QString &person, const QString &friendV, const QString &message, const QString &achievement,
             const QString &activity, const QString &content, const QString &fan, const QString &forum,
-            const QString &knowledgebase, const QString &event, const QString &comment, const QString &registerUrl)
+            const QString &knowledgebase, const QString &event, const QString &comment, const QString &registerUrl,
+            const QString &additionalAgentInformation)
         : m_baseUrl(baseUrl), m_icon(icon), m_name(name)
         , m_personVersion(person)
         , m_friendVersion(friendV)
@@ -165,6 +155,7 @@ public:
         , m_commentVersion(comment)
         , m_registerUrl(registerUrl)
         , m_internals(internals)
+        , m_additionalAgentInformation(additionalAgentInformation)
     {
         if (m_baseUrl.isEmpty()) {
             return;
@@ -197,7 +188,7 @@ Provider::Provider(PlatformDependent *internals, const QUrl &baseUrl, const QStr
                    const QString &activity, const QString &content, const QString &fan, const QString &forum,
                    const QString &knowledgebase, const QString &event, const QString &comment)
     : d(new Private(internals, baseUrl, name, icon, person, friendV, message, achievement, activity, content,
-                    fan, forum, knowledgebase, event, comment, QString()))
+                    fan, forum, knowledgebase, event, comment, QString(), QString()))
 {
 }
 
@@ -206,7 +197,17 @@ Provider::Provider(PlatformDependent *internals, const QUrl &baseUrl, const QStr
                    const QString &activity, const QString &content, const QString &fan, const QString &forum,
                    const QString &knowledgebase, const QString &event, const QString &comment, const QString &registerUrl)
     : d(new Private(internals, baseUrl, name, icon, person, friendV, message, achievement, activity, content,
-                    fan, forum, knowledgebase, event, comment, registerUrl))
+                    fan, forum, knowledgebase, event, comment, registerUrl, QString()))
+{
+}
+
+Provider::Provider(PlatformDependent *internals, const QUrl &baseUrl, const QString &name, const QUrl &icon,
+                   const QString &person, const QString &friendV, const QString &message, const QString &achievement,
+                   const QString &activity, const QString &content, const QString &fan, const QString &forum,
+                   const QString &knowledgebase, const QString &event, const QString &comment, const QString &registerUrl,
+                   const QString &additionalAgentInformation)
+    : d(new Private(internals, baseUrl, name, icon, person, friendV, message, achievement, activity, content,
+                    fan, forum, knowledgebase, event, comment, registerUrl, additionalAgentInformation))
 {
 }
 
@@ -248,6 +249,14 @@ void Provider::setEnabled(bool enabled)
     d->m_internals->enableProvider(d->m_baseUrl, enabled);
 }
 
+void Provider::setAdditionalAgentInformation(const QString& additionalInformation) {
+    d->m_additionalAgentInformation = additionalInformation;
+}
+
+QString Provider::additionalAgentInformation() const {
+    return d->m_additionalAgentInformation;
+}
+
 QString Provider::name() const
 {
     return d->m_name;
@@ -456,7 +465,7 @@ ListJob<Achievement> *Provider::requestAchievements(const QString &contentId, co
         return nullptr;
     }
 
-    QUrl url = createUrl(QStringLiteral("achievements/content/") + contentId + achievementId);
+    QUrl url = createUrl(QLatin1String("achievements/content/") + contentId + achievementId);
     QUrlQuery q(url);
     q.addQueryItem(QStringLiteral("user_id"), userId);
     url.setQuery(q);
@@ -477,12 +486,14 @@ ItemPostJob<Achievement> *Provider::addNewAchievement(const QString &contentId,
     postParameters.insert(QLatin1String("explanation"), newAchievement.explanation());
     postParameters.insert(QLatin1String("points"), QString::number(newAchievement.points()));
     postParameters.insert(QLatin1String("image"), newAchievement.image().toLocalFile());
-    foreach (const QString &dependency, newAchievement.dependencies()) {
+    const auto dependenciesList = newAchievement.dependencies();
+    for (const QString &dependency : dependenciesList) {
         postParameters.insert(QString::fromLatin1("dependencies[%1]").arg(QString::number(i++)), dependency);
     }
 
     postParameters.insert(QLatin1String("type"), Achievement::achievementTypeToString(newAchievement.type()));
-    foreach (const QString &option, newAchievement.options()) {
+    const auto optionsList = newAchievement.options();
+    for (const QString &option : optionsList) {
         postParameters.insert(QString::fromLatin1("options[%1]").arg(QString::number(j++)), option);
     }
 
@@ -510,12 +521,14 @@ PutJob *Provider::editAchievement(const QString &contentId, const QString &achie
     postParameters.insert(QLatin1String("explanation"), achievement.explanation());
     postParameters.insert(QLatin1String("points"), QString::number(achievement.points()));
     postParameters.insert(QLatin1String("image"), achievement.image().toLocalFile());
-    foreach (const QString &dependency, achievement.dependencies()) {
+    const auto dependenciesList = achievement.dependencies();
+    for (const QString &dependency : dependenciesList) {
         postParameters.insert(QString::fromLatin1("dependencies[%1]").arg(QString::number(i++)), dependency);
     }
 
     postParameters.insert(QLatin1String("type"), Achievement::achievementTypeToString(achievement.type()));
-    foreach (const QString &option, achievement.options()) {
+    const auto optionsList = achievement.options();
+    for (const QString &option : optionsList) {
         postParameters.insert(QString::fromLatin1("options[%1]").arg(QString::number(j++)), option);
     }
 
@@ -615,7 +628,7 @@ QMap<QString, QString> projectPostParameters(const Project &project)
         postParameters.insert(QLatin1String("url"), project.url());
     }
     if (!project.developers().isEmpty()) {
-        postParameters.insert(QLatin1String("developers"), project.developers().join(QLatin1String("\n")));
+        postParameters.insert(QLatin1String("developers"), project.developers().join(QLatin1Char('\n')));
     }
     if (!project.version().isEmpty()) {
         postParameters.insert(QLatin1String("version"), project.version());
@@ -1001,7 +1014,7 @@ ListJob<Message> *Provider::requestMessages(const Folder &folder, Message::Statu
         return nullptr;
     }
 
-    QUrl url = createUrl(QStringLiteral("message/") + folder.id());
+    QUrl url = createUrl(QLatin1String("message/") + folder.id());
     QUrlQuery q(url);
     q.addQueryItem(QStringLiteral("status"), QString::number(status));
     url.setQuery(q);
@@ -1093,22 +1106,23 @@ ListJob<Content> *Provider::searchContents(const Category::List &categories, con
     QUrl url = createUrl(QStringLiteral("content/data"));
     QUrlQuery q(url);
     QStringList categoryIds;
-    foreach (const Category &category, categories) {
+    categoryIds.reserve(categories.count());
+    for (const Category &category : categories) {
         categoryIds.append(category.id());
     }
-    q.addQueryItem(QStringLiteral("categories"), categoryIds.join(QLatin1String("x")));
+    q.addQueryItem(QStringLiteral("categories"), categoryIds.join(QLatin1Char('x')));
 
     QStringList distributionIds;
-    foreach (const Distribution &distribution, distributions) {
+    for (const Distribution &distribution : distributions) {
         distributionIds.append(QString(distribution.id()));
     }
-    q.addQueryItem(QStringLiteral("distribution"), distributionIds.join(QLatin1String(",")));
+    q.addQueryItem(QStringLiteral("distribution"), distributionIds.join(QLatin1Char(',')));
 
     QStringList licenseIds;
-    foreach (const License &license, licenses) {
+    for (const License &license : licenses) {
         licenseIds.append(QString(license.id()));
     }
-    q.addQueryItem(QStringLiteral("license"), licenseIds.join(QLatin1String(",")));
+    q.addQueryItem(QStringLiteral("license"), licenseIds.join(QLatin1Char(',')));
 
     if (!person.isEmpty()) {
         q.addQueryItem(QStringLiteral("user"), person);
@@ -1269,6 +1283,7 @@ PostJob *Provider::deletePreviewImage(const QString &contentId, const QString &p
     return new PostJob(d->m_internals, postRequest.request(), postRequest.data());
 }
 
+#if ATTICA_BUILD_DEPRECATED_SINCE(0, 2)
 PostJob *Provider::voteForContent(const QString &contentId, bool positiveVote)
 {
     if (!isValid()) {
@@ -1280,6 +1295,7 @@ PostJob *Provider::voteForContent(const QString &contentId, bool positiveVote)
     //qCDebug(ATTICA) << "vote: " << positiveVote;
     return new PostJob(d->m_internals, createRequest(QLatin1String("content/vote/") + contentId), postParameters);
 }
+#endif
 
 PostJob *Provider::voteForContent(const QString &contentId, uint rating)
 {
@@ -1317,7 +1333,7 @@ ListJob<Person> *Provider::requestFans(const QString &contentId, uint page, uint
         return nullptr;
     }
 
-    QUrl url = createUrl(QStringLiteral("fan/data/") + contentId);
+    QUrl url = createUrl(QLatin1String("fan/data/") + contentId);
     QUrlQuery q(url);
     q.addQueryItem(QStringLiteral("contentid"), contentId);
     q.addQueryItem(QStringLiteral("page"), QString::number(page));
@@ -1596,6 +1612,18 @@ QNetworkRequest Provider::createRequest(const QUrl &url)
 {
     QNetworkRequest request(url);
     request.setHeader(QNetworkRequest::ContentTypeHeader,QStringLiteral("application/x-www-form-urlencoded"));
+
+    QString agentHeader;
+    if (QCoreApplication::instance()) {
+        agentHeader = QString::fromLocal8Bit("%1/%2").arg(QCoreApplication::instance()->applicationName(), QCoreApplication::instance()->applicationVersion());
+    } else {
+        agentHeader = QString::fromLocal8Bit("Attica/%1").arg(QLatin1String(LIBATTICA_VERSION_STRING));
+    }
+    if (!d->m_additionalAgentInformation.isEmpty()) {
+        agentHeader = QString::fromLocal8Bit("%1 (+%2)").arg(agentHeader, d->m_additionalAgentInformation);
+    }
+    request.setHeader(QNetworkRequest::UserAgentHeader, agentHeader);
+
     if (!d->m_credentialsUserName.isEmpty()) {
         request.setAttribute((QNetworkRequest::Attribute) BaseJob::UserAttribute, QVariant(d->m_credentialsUserName));
         request.setAttribute((QNetworkRequest::Attribute) BaseJob::PasswordAttribute, QVariant(d->m_credentialsPassword));
